from LoreleiClient.Views.Modules.Common import Component
from LoreleiClient.UIObjects.Input import Box, LabelRightAligned, Label


class StatisticsComponent(Component):

    def __init__(self, x, y, character):
        self.character = character
        super(StatisticsComponent, self).__init__(x, y, self.setup)

    def setup(self):
        self.add_ui_object(Box(self.x, self.y, 886, 200, 3, "Statistics", fontSize=24))
        spacing = 28
        labelX = self.x + 64
        intValueX = labelX + 88

        viewDict = self.character.attributes.getViewDict()
        for section, valueDict in viewDict.iteritems():
            y = self.y + 4
            self.add_ui_object(LabelRightAligned(labelX, y, 80, 28, section, fontSize=24, underlined=True))
            y += spacing
            for key, value in valueDict.iteritems():
                self.add_ui_object(LabelRightAligned(labelX, y, 80, 20, key + " : ", fontSize=24))
                self.add_ui_object(Label(labelX + 80, y, value, fontSize=24))
                y += spacing
            labelX += 200


    def updateComponent(self):
        self.removeAll()
        self.setup()


from LoreleiClient.Views.Common import UIView
from LoreleiLib.Packets.View import ViewType
from LoreleiClient.UIObjects.Input import Label


class GameView(UIView):

    def __init__(self, connection):
        super(GameView, self).__init__(self.makeView, ViewType.Game, connection)

    def makeView(self):
        if self.connection is not None:
            self.add_ui_object(Label(20, 20, "GAME MODE", 42))


import pygame as pg
from LoreleiClient.Settings.Screen import ScreenSettings
from LoreleiClient.Images.Manager import ImageManager
from threading import Timer
from LoreleiLib.StringHelpers import wrap_multi_line
pg.font.init()

COLOR_INACTIVE = pg.Color('seashell1')
COLOR_ACTIVE = pg.Color('yellow2')


class Label(object):

    w_padding = 4
    h_padding = 4

    def __init__(self, x, y, text, fontSize=18, underlined=False):
        text = str(text)
        self.text = text
        self.color = COLOR_ACTIVE
        self.font = pg.font.Font(None, fontSize)
        self.font.set_underline(underlined)
        self.rect = pg.Rect(x, y, self.font.size(self.text)[0], self.font.size(self.text)[1])
        self.txt_surface = self.font.render(text, True, self.color)

    def draw(self, screen):
        # Blit the text.
        screen.blit(self.txt_surface, (self.rect.x + Label.w_padding, self.rect.y + Label.h_padding))

    def get_width(self):
        return self.font.size(self.text)[0]


class LabelRightAligned(Label):

    def __init__(self, x, y, w, h, text, fontSize=18, underlined=False):
        self.font = pg.font.Font(None, fontSize)
        size = self.font.size(text)

        x = (x + w) - (size[0] + Label.w_padding)
        super(LabelRightAligned, self).__init__(x, y, text, fontSize, underlined)


class Box(object):

    def __init__(self, x, y, w, h, box_width, name="", fontSize=18):
        self.rect = pg.Rect(x, y, w, h)
        self.boxWidth = box_width
        self.font = pg.font.Font(None, fontSize)
        self.fontSize = fontSize
        self.name = name
        self.txt_surface = self.font.render(name, True, COLOR_ACTIVE)

    def draw(self, screen):
        pg.draw.rect(screen, COLOR_INACTIVE, self.rect, self.boxWidth)
        textRect = pg.Rect(self.rect.x + 16, self.rect.y - (self.fontSize / 3), self.font.size(self.name)[0], self.font.size(self.name)[1])
        textRectBackground = pg.Rect(textRect.x - 6, textRect.y, textRect.w + 12, textRect.h)

        if self.name != "":
            # Blit the text.
            pg.draw.rect(screen, (50, 50, 50), textRectBackground)
            screen.blit(self.txt_surface, textRect)


class ParagraphBox(Box):

    def __init__(self, x, y, w, h, box_width, name="", observable=None, fontSize=18):
        super(ParagraphBox, self).__init__(x, y, w, h, box_width, name, fontSize)
        self.text = str(observable.value)
        self.observable = observable
        if observable is not None:
            self.observable.onChangeCallback = self.observableCallback
        self.lines = wrap_multi_line(self.text, self.font, self.rect.w-8)

    def observableCallback(self, value):
        self.text = str(value)
        self.lines = wrap_multi_line(self.text, self.font, self.rect.w-8)

    def draw(self, screen):
        pg.draw.rect(screen, (50, 50, 50), self.rect)
        super(ParagraphBox, self).draw(screen)
        y = self.rect.top + 8
        x = self.rect.left + 4
        for line in self.lines:
            if y + self.fontSize < self.rect.y + self.rect.h:
                line_size = self.font.size(line)
                txt_area = self.font.render(line, True, COLOR_INACTIVE)
                screen.blit(txt_area, (x, y, line_size[0], line_size[1]))
                y += self.fontSize


class ObservableValue:

    def __init__(self, value, callback=None):
        # type: (object, classmethod) -> None
        self.value = value
        self.onChangeCallback = callback

    def setValue(self, value):
        if self.value != value:
            self.value = value
            if self.onChangeCallback is not None:
                self.onChangeCallback(self.value)


class IntValueBox:

    def __init__(self, x, y, w, h, observable, fontSize=18):
        # type: (int, int, int, int, ObservableValue) -> None
        self.observable = observable
        self.observable.onChangeCallback = self.valueUpdated
        self.text = None
        self.textOffset = (Label.w_padding, Label.h_padding)
        self.color = COLOR_ACTIVE
        self.font = pg.font.Font(None, fontSize)
        self.rect = pg.Rect(x, y, w, h)
        self.valueUpdated(int(observable.value))

    def valueUpdated(self, value):
        # type: (int) -> None
        self.text = str(value)
        size = self.font.size(self.text)
        self.textOffset = ((self.rect.w - size[0]) / 2, (self.rect.h - size[1]) / 2)

    def draw(self, screen):
        txt_surface = self.font.render(self.text, True, self.color)
        screen.blit(txt_surface, (self.rect.x + self.textOffset[0], self.rect.y + self.textOffset[1]))
        pg.draw.rect(screen, COLOR_INACTIVE, self.rect, 2)


class Button(object):

    w_padding = 8
    h_padding = 4
    hover_offset = 2

    def __init__(self, x, y, w, h, text, callback, fontSize=18):
        self.hover = False
        self.color = COLOR_INACTIVE
        self.font = pg.font.Font(None, fontSize)
        font_render_size = self.font.size(text)
        self.rect = pg.Rect(x, y, font_render_size[0] + Button.w_padding*2, font_render_size[1] + Button.h_padding*2)
        self.hover_rect = pg.Rect(x-Button.hover_offset, y-Button.hover_offset, self.rect.w, self.rect.h)
        self.callback = callback
        self.text = text

        font_area = self.font.size(text)

        self.txt_surface = self.font.render(text, True, self.color)

    def handle_event(self, event):
        if event.type == pg.MOUSEMOTION:
            if self.rect.collidepoint(pg.mouse.get_pos()) or self.hover_rect.collidepoint(pg.mouse.get_pos()):
                self.hover = True
            else:
                self.hover = False
        elif event.type == pg.MOUSEBUTTONDOWN:
            # If the user clicked on the input_box rect.
            if self.rect.collidepoint(event.pos) :
                # Toggle the active variable.
                self.callback()

                self.color = COLOR_ACTIVE
                Timer(0.1, self.reset).start()

    def reset(self):
        self.color = COLOR_INACTIVE

    def draw(self, screen):
        if self.hover:
            # Blit the text.
            screen.blit(self.txt_surface, (self.hover_rect.x + Button.w_padding, self.hover_rect.y + Button.h_padding))
            # Blit the rect.
            pg.draw.rect(screen, self.color, self.hover_rect, 2)

        else:
            # Blit the text.
            screen.blit(self.txt_surface, (self.rect.x+Button.w_padding, self.rect.y+Button.h_padding))
            # Blit the rect.
            pg.draw.rect(screen, self.color, self.rect, 2)


class ConditionalUI(object):

    def __init__(self, uiObject, drawConditional=None, interactConditional=None):
        self.uiObject = uiObject
        self.drawCondition = drawConditional
        self.interactCondition = interactConditional

    def draw(self, screen):
        if self.drawCondition is None:
            self.uiObject.draw(screen)
        else:
            if self.drawCondition():
                self.uiObject.draw(screen)

    def handle_event(self, event):
        if self.interactCondition is None:
            self.uiObject.handle_event(event)
        else:
            if self.interactCondition():
                self.uiObject.handle_event(event)

    def update(self):
        if hasattr(self.uiObject, "update"):
            self.uiObject.update()


class Dropdown(object):

    w_padding = 4
    h_padding = 4

    # x, y : Coordinates to draw
    # default_text : Text to display when an option isn't selected
    # options : The options to select from
    # displayValue : The attribute to show in the dropdown
    def __init__(self, x, y, defaultText, options, displayValue, selectCallback, minWidth=None, fontSize=18):
        self.open = False
        self.option = None
        self.options = options
        self.defaultText = defaultText
        self.dropdownRects = []
        self.dropdownSquare = None
        self.selectCallback = selectCallback
        self.displayValue = displayValue
        self.font = pg.font.Font(None, fontSize)
        self.minWidth = minWidth
        size = self.getLargestItemSize()
        self.rect = pg.Rect(x, y, size[0]+(Dropdown.w_padding*2), size[1] + (Dropdown.h_padding*2))

    def getLargestItemSize(self):
        largest = self.font.size(self.defaultText)
        for option in self.options:
            size = self.font.size(getattr(option, self.displayValue))
            if size[0] > largest[0]:
                largest = size

        if self.minWidth > largest[0]:
            largest = (self.minWidth, largest[1])
        return largest

    def buildRects(self):
        self.dropdownRects = []
        if self.open:
            y = self.rect.y + self.rect.h

            for option in self.options:
                next_rect = pg.Rect(self.rect.x, y, self.rect.w, self.rect.h)
                self.dropdownRects.append(next_rect)
                y = next_rect.y + next_rect.h

    def handle_event(self, event):
        if event.type == pg.MOUSEBUTTONDOWN:
            pos = event.pos
            if not self.open:
                if self.rect.collidepoint(pos):
                    self.open = True
                    self.buildRects()
            else:
                for i in range(0, len(self.dropdownRects)):
                    rect = self.dropdownRects[i]
                    if rect.collidepoint(pos):
                        self.setOption(self.options[i])

                self.open = False
                self.buildRects()

    def draw(self, screen):
        if len(self.options) == 0:
            return

        selected_text = self.defaultText
        if self.option is not None:
            selected_text = getattr(self.option, self.displayValue)
        self.drawOption(screen, self.rect, selected_text, COLOR_ACTIVE, COLOR_INACTIVE)

        for i in range(0, len(self.dropdownRects)):
            rect = self.dropdownRects[i]
            option = self.options[i]
            self.drawOption(screen, rect, getattr(option, self.displayValue), COLOR_INACTIVE, COLOR_ACTIVE)

    def setOption(self, option):
        self.option = option
        self.selectCallback(option)

    def drawOption(self, screen, rect, text, text_color, box_color):
        pg.draw.rect(screen, (50, 50, 50), rect)
        # Blit the text.
        screen.blit(self.font.render(text, True, text_color), (rect.x + Dropdown.w_padding, rect.y + Dropdown.h_padding))
        # Blit the rect.
        pg.draw.rect(screen, box_color, rect, 2)


class Image(object):

    def __init__(self, x, y, imageName):
        self.x = x
        self.y = y
        image = ImageManager().Images[imageName]
        if image is None:
            print "Could not load image for resource name,", imageName
        else:
            self.image = image

    def draw(self, screen):
        if self.image is not None:
            screen.blit(self.image, (self.x, self.y))


class InputBox(object):

    w_padding = 4
    h_padding = 4

    def __init__(self, x, y, w, h, text='', prefix='', fontSize=18, enter_callback=None):
        self.rect = pg.Rect(x, y, w, h)
        self.color = COLOR_INACTIVE
        self.font = pg.font.Font(None, fontSize)
        self.prefix = prefix
        self.text = text
        self.enter_callback = enter_callback
        self.txt_surface = self.font.render(prefix + text, True, self.color)
        self.active = False

    def handle_event(self, event):
        if event.type == pg.MOUSEBUTTONDOWN:
            # If the user clicked on the input_box rect.
            if self.rect.collidepoint(event.pos):
                # Toggle the active variable.
                self.set_active(True)
            else:
                self.set_active(False)

        if event.type == pg.KEYDOWN:
            if self.active:
                if event.key == pg.K_RETURN:
                    if self.enter_callback is not None:
                        self.enter_callback(self.text)
                elif event.key == pg.K_BACKSPACE:
                    self.text = self.text[:-1]
                elif event.key == pg.K_TAB:
                    pass
                else:
                    self.text += event.unicode
                # Re-render the text.
                self.txt_surface = self.font.render(self.prefix + self.text, True, self.color)

    def set_active(self, active):
        self.active = active
        # Change the current color of the input box.
        self.color = COLOR_ACTIVE if self.active else COLOR_INACTIVE

    def update(self):
        # Resize the box if the text is too long.
        width = max(200, self.txt_surface.get_width()+(InputBox.w_padding*2))
        self.rect.w = width

    def draw(self, screen):
        pg.draw.rect(screen, (50, 50, 50), self.rect)
        # Blit the text.
        screen.blit(self.txt_surface, (self.rect.x+InputBox.w_padding, self.rect.y+InputBox.h_padding))
        # Blit the rect.
        pg.draw.rect(screen, self.color, self.rect, 2)


class PasswordInput(InputBox):

    def __init__(self, x, y, w, h, text='', prefix='', fontSize=18, enter_callback=None):
        super(PasswordInput, self).__init__(x, y, w, h, text, prefix, fontSize, enter_callback)
        self.value = ''
        self.txt_surface = self.font.render(self.value, True, self.color)

    def draw(self, screen):
        self.value = ''
        for i in range(0, len(self.text)):
            self.value += '*'

        self.txt_surface = self.font.render(self.value, True, self.color)
        # Blit the text.
        screen.blit(self.txt_surface, (self.rect.x+5, self.rect.y+5))
        # Blit the rect.
        pg.draw.rect(screen, self.color, self.rect, 2)


class Notification(object):

    def __init__(self, text, callback=None, button_text="Ok", text_color=COLOR_INACTIVE, fontSize=18, hideButton=False):
        self.padding = 10
        self.color = COLOR_INACTIVE
        self.font = pg.font.Font(None, fontSize)
        self.callback = callback
        self.text = text
        self.text_color = text_color
        self.open = True
        self.hideButton = hideButton
        self.txt_surface = self.font.render(text, True, self.text_color)
        font_area = self.font.size(text)
        self.rect = pg.Rect(ScreenSettings.SCREEN_WIDTH/2 - font_area[0]/2 - self.padding, ScreenSettings.SCREEN_HEIGHT/2 - font_area[1]/2 - self.padding, font_area[0] + self.padding*2, font_area[1] + self.padding*2)

        button_text_size = self.font.size(button_text)
        self.button = Button(self.rect.x + self.rect.w - (button_text_size[0] + 20), self.rect.y + self.rect.h + 10, button_text_size[0] + (Button.w_padding*2), button_text_size[1] + (Button.h_padding*2), button_text, self.call_callback)

        # Find text size
        # Make Rect around it
        # Make Button

    def call_callback(self):
        print "Notification close?"
        self.open = False
        if self.callback is not None:
            self.callback()

    def draw(self, screen):
        if self.open:
            pg.draw.rect(screen, (50,50,50), self.rect)
            # Blit the text.
            screen.blit(self.txt_surface, (self.rect.x+self.padding, self.rect.y+self.padding))
            # Blit the rect.
            pg.draw.rect(screen, self.color, self.rect, 2)

            if not self.hideButton:
                self.button.draw(screen)

    def handle_event(self, event):
        if not self.hideButton:
            self.button.handle_event(event)



from twisted.internet import reactor
from twisted.internet.protocol import ReconnectingClientFactory, Protocol
from twisted.internet.task import LoopingCall
from LoreleiClient.UI import UI
from LoreleiClient.Images.Manager import ImageManager
from LoreleiLib.Packets.Common import Packet, PacketSet
import cPickle
import pygame


class Connection:

    def __init__(self, transport=None):
        self.transport = transport

    def sendLine(self, line):
        if self.transport is not None:
            if isinstance(line, Packet):
                line = line.serialize()
            self.transport.write(line)


class Listener(Protocol):

    def __init__(self):
        self.dataObject = ""

    def connectionMade(self):
        ui.set_connection(Connection(self.transport))

    def dataReceived(self, data):
        try:
            data = cPickle.loads(data)
        except Exception as err:
            self.dataObject += data
            try:
                data = cPickle.loads(self.dataObject)
                self.dataObject = ""
                # print "Unpickled Large Object"
            except Exception as err2:
                pass
                # print "Object not finished"

        if isinstance(data, PacketSet):
            for packet in data.packets:
                self.handlePacket(packet)
        else:
            self.handlePacket(data)

    def handlePacket(self, packet):
        ui.handle_data(packet)


class ClientFactory(ReconnectingClientFactory):

    def startedConnecting(self, connector):
        print 'Started to connect.'

    def buildProtocol(self, addr):
        print 'Connected.'
        self.resetDelay()
        return Listener()

    def clientConnectionLost(self, connector, reason):
        if ui.running:
            print 'Lost connection.  Reason:', reason
            print 'Retrying connectiong...'
            connector.connect()
        ReconnectingClientFactory.clientConnectionLost(self, connector, reason)

    def clientConnectionFailed(self, connector, reason):
        print 'Connection failed. Reason:', reason
        ReconnectingClientFactory.clientConnectionFailed(self, connector,
                                                         reason)


pygame.init()
ui = UI(Connection())


def loadUp():
    ImageManager()
    pygame.display.set_icon(ImageManager.Images["icon"])


def ui_tick():
    if not ui.game_tick():
        reactor.stop()


def main():
    loadUp()
    reactor.connectTCP("104.0.149.42", 8123, ClientFactory())
    # reactor.connectTCP("localhost", 8123, ClientFactory())

    tick = LoopingCall(ui_tick)
    tick.start(1.0 / ui.DESIRED_FPS)
    try:
        reactor.run()
    except Exception as err:
        if err != "display Surface quit":
            print "REACTOR RUN FAILURE", err

if __name__ == "__main__":
    main()


from LoreleiClient.UIObjects.Input import Box, Label, Image, ParagraphBox, ObservableValue
from LoreleiClient.Views.Modules.Common import Component
from LoreleiClient.Images.Manager import ImageManager
from LoreleiLib.Objects import EquipmentSlot, Weapon, Shield
from LoreleiLib.StringHelpers import wrap_multi_line
from collections import OrderedDict
import pygame as pg


class EquipmentHoverover:

    def __init__(self, x, y, item, fontSize=18):
        self.x = x
        self.y = y
        self.font = pg.font.Font(None, fontSize)
        self.fontSize = fontSize
        self.item = item
        self.stats = self.item.getTooltipInfo()
        self.box = None
        self.observable = None
        self.setup()

    def setup(self):
        size = [50, 0]
        totalText = ""
        for key, value in self.stats.iteritems():
            if isinstance(value, OrderedDict):
                for key2, value2 in value.iteritems():
                    nextText = "\t" + key2 + " : " + str(value2) + "\n"
                    nextSize = self.font.size(nextText)
                    if nextSize[0] > size[0]:
                        size[0] = nextSize[0]
                    size[1] += self.fontSize
                    totalText += nextText
            else:
                nextText = key + " : " + str(value) + "\n"
                nextSize = self.font.size(nextText)
                if nextSize[0] > size[0]:
                    size[0] = nextSize[0]
                size[1] += self.fontSize
                totalText += nextText

        if size[0] > 200:
            size[0] = 200
        size[1] = (self.fontSize * len(wrap_multi_line(totalText, self.font, size[0]))) + 16

        self.observable = ObservableValue(totalText)
        self.box = ParagraphBox(self.x, self.y, size[0], size[1], 3, self.item.name, self.observable, self.fontSize)

    def move(self, x, y):
        self.box.rect.x = x
        self.box.rect.y = y

    def draw(self, screen):
        self.box.draw(screen)


class EquipmentItemComponent:

    def __init__(self, x, y, slot, item):
        self.x = x
        self.y = y
        self.slot = slot
        self.item = item
        self.label = None
        self.image = None
        self.hover = False
        self.hoverRect = None
        self.hoverover = None
        self.setup()

    def setup(self):
        slotname = self.slot.name.lower()
        itemname = "<empty>"

        if self.slot == EquipmentSlot.Off_Hand:
            if isinstance(self.item, Weapon) and not isinstance(self.item, Shield):
                slotname = EquipmentSlot.Main_Hand.name.lower()

        if self.item is not None:
            itemname = self.item.name

        # Draw Slot Image
        self.image = Image(self.x + 10, self.y, slotname)
        # Draw Text
        self.label = Label(self.x + 52, self.y + 7, itemname, fontSize=18)

        if self.item is not None:
            self.hoverRect = pg.Rect(self.x, self.y, (self.label.rect.x + self.label.rect.w - self.x) + 32, 32)
            self.hoverover = EquipmentHoverover(self.x, self.y, self.item)

    def draw(self, screen):
        self.image.draw(screen)
        self.label.draw(screen)

        if self.hover:
            # draw the hover over
            if self.hoverover is not None:
                self.hoverover.draw(screen)

    def handle_event(self, event):
        if event.type == pg.MOUSEMOTION:
            if self.hoverRect is not None:
                if self.hoverRect.collidepoint(pg.mouse.get_pos()):
                    pos = pg.mouse.get_pos()
                    self.hoverover.move(pos[0]+20, pos[1]+20)
                    self.hover = True
                else:
                    self.hover = False


class EquipmentComponent(Component):

    def __init__(self, x, y, character, editable=False):
        self.character = character
        self.imageManager = ImageManager()
        self.editable = editable # If we want the player to be able to change equipment in this view
        super(EquipmentComponent, self).__init__(x, y, self.setup)

    def setup(self):
        spacing = 32
        x = self.x
        y = self.y
        self.add_ui_object(Box(x, y, 300, 374, 3, "Equipment")) # 32 * 11 = 352

        items = []

        y += 10
        for slot, item in self.character.equipment.getAllItems().iteritems():
            items.append(EquipmentItemComponent(x, y, slot, item))
            y += spacing

        for item in reversed(items):
            self.add_ui_object(item)

    def updateComponent(self):
        self.removeAll()
        self.setup()


from LoreleiClient.Views.Common import UIView
from LoreleiClient.UIObjects.Input import Label, Dropdown, IntValueBox, LabelRightAligned, Box, \
    ObservableValue, ParagraphBox, Button, ConditionalUI, Image, Notification
from LoreleiClient.Images.Manager import ImageManager
from LoreleiLib.Packets.CharacterCreation import OptionsRequest, RaceSelectionRequest, \
    ClassSelectionRequest, CharacterCheckResponse, CreationOptionsResponse, CharacterFinishRequest, \
    CharacterFinishResponse
from LoreleiClient.Settings.Screen import ScreenSettings
from LoreleiClient.Views.Modules.Attributes import AttributeComponent
from LoreleiClient.Views.Modules.Equipment import EquipmentComponent
from LoreleiClient.Views.Modules.Statistics import StatisticsComponent
from LoreleiClient.Views.Game import GameView
from LoreleiLib.Character import Character, CharacterClass, CharacterRace
from LoreleiLib.Packets.View import ViewType


class CharacterCreationView(UIView):

    def __init__(self, connection=None, character=None):
        # Type: () -> None
        self.notification = None
        self.classes = [] # type: list<CharacterClass>
        self.races = [] # type: list<CharacterRace>
        if character is None:
            character = Character('', None, None)
        self.character = character
        self.attributes = AttributeComponent(54, 204, self.character) # 224 - 140 = 64
        self.equipment = EquipmentComponent(640, 70, self.character) # 444 bot, 940 right
        self.statistics = StatisticsComponent(54, 464, self.character) # 940-54 = 886, 464-240 = 224
        self.strength = ObservableValue(self.character.attributes.strength)
        self.constitution = ObservableValue(self.character.attributes.constitution)
        self.dexterity = ObservableValue(self.character.attributes.dexterity)
        self.agility = ObservableValue(self.character.attributes.agility)
        self.wisdom = ObservableValue(self.character.attributes.wisdom)
        self.intelligence = ObservableValue(self.character.attributes.intelligence)
        self.charisma = ObservableValue(self.character.attributes.charisma)
        self.raceDescription = ObservableValue('Select a Race to see its description')
        self.classDescription = ObservableValue('Select a Class to see its description')

        if self.character.character_race is not None:
            self.raceDescription.setValue(self.character.character_race.description)
        if self.character.character_class is not None:
            self.classDescription.setValue(self.character.character_class.description)

        super(CharacterCreationView, self).__init__(self.makeView, ViewType.CharacterCreation, connection)

    def makeView(self):
        if self.connection is not None:
            self.add_ui_object(Label(20, 20, "Character Creation", 42))
            self.add_ui_object(self.attributes)
            self.add_ui_object(self.equipment)
            self.add_ui_object(self.statistics)

            self.makeDropDowns(124, 102)

            self.add_ui_object(ParagraphBox(260, 70, 350, 120, 3, "Race Description", self.raceDescription, fontSize=24)) # 260 + 350 = 610
            self.add_ui_object(ParagraphBox(260, 200, 350, 244, 3, "Class Description", self.classDescription, fontSize=24))

            self.add_ui_object(ConditionalUI(Button(ScreenSettings.SCREEN_WIDTH-90, ScreenSettings.SCREEN_HEIGHT-50, 40, 24, "Finish", self.finish, fontSize=24), self.showNextButton))

    def makeDropDowns(self, x, y):
        self.add_ui_object(LabelRightAligned(x-88, y, 80, 20, "Race :", fontSize=24))
        self.add_ui_object(LabelRightAligned(x-88, y+30, 80, 20, "Class :", fontSize=24))
        classDropdown = Dropdown(x, y+30, "Select", self.classes, 'name', self.setClass, minWidth=100, fontSize=24)
        classDropdown.option = self.character.character_class

        raceDropdown = Dropdown(x, y, "Select", self.races, 'name', self.setRace, minWidth=100, fontSize=24)
        raceDropdown.option = self.character.character_race
        self.add_ui_object(classDropdown)
        self.add_ui_object(raceDropdown)

    def setRace(self, value):
        self.character.changeRace(value)
        self.rebuildCharacter()
        self.connection.sendLine(RaceSelectionRequest(value))

    def setClass(self, value):
        self.character.character_class = value
        self.rebuildCharacter()
        self.connection.sendLine(ClassSelectionRequest(value))

    def rebuildCharacter(self):
        self.character.buildStats()
        self.strength.setValue(self.character.attributes.strength)
        self.constitution.setValue(self.character.attributes.constitution)
        self.dexterity.setValue(self.character.attributes.dexterity)
        self.agility.setValue(self.character.attributes.agility)
        self.wisdom.setValue(self.character.attributes.wisdom)
        self.intelligence.setValue(self.character.attributes.intelligence)
        self.charisma.setValue(self.character.attributes.charisma)

        if self.character.character_race is not None:
            self.raceDescription.setValue(
                self.character.character_race.description)
        if self.character.character_class is not None:
            self.classDescription.setValue(self.character.character_class.getFullDescription())

    def getData(self):
        self.connection.sendLine(OptionsRequest())
        self.rebuildCharacter()

    def showNextButton(self):
        return self.character.character_class is not None and self.character.character_race is not None

    def notification_callback(self):
        self.remove_ui_object(self.notification)
        self.notification = None

    def handle_data(self, data):
        if isinstance(data, CreationOptionsResponse):
            self.races = data.races
            self.classes = data.classes
        elif isinstance(data, CharacterCheckResponse):
            self.removeAll()
            self.character = data.character
            self.attributes = AttributeComponent(54, 204, self.character)  # 224 - 140 = 64
            self.equipment = EquipmentComponent(640, 70, self.character)  # 444 bot, 940 right
            self.statistics = StatisticsComponent(54, 464, self.character)  # 940-54 = 886, 464-240 = 224
            self.makeView()
        elif isinstance(data, CharacterFinishResponse):
            print "Got Character Finish Response"
            if data.success:
                return GameView(self.connection)
            else:
                self.add_ui_object(Notification(data.message, self.notification_callback))

        if len(self.races) > 0 and len(self.classes) > 0:
            self.makeView()

    def finish(self):
        self.connection.sendLine(CharacterFinishRequest())


import pygame
import os

class ImageManager:

    Images = {}

    def __init__(self):
        if len(ImageManager.Images) == 0:
            self.load()

    def load(self):
        root = os.path.join(os.path.dirname(os.path.realpath(__file__)), "Data")
        ImageManager.Images["icon"] = self.loadImage(root, "crest.png")
        ImageManager.Images["main_hand"] = self.loadImage(root, "sword_small.png")
        ImageManager.Images["off_hand"] = self.loadImage(root, "shield_small.png")
        ImageManager.Images["head"] = self.loadImage(root, "helm_small.png")
        ImageManager.Images["chest"] = self.loadImage(root, "chest_small.png")
        ImageManager.Images["hands"] = self.loadImage(root, "glove_small.png")
        ImageManager.Images["legs"] = self.loadImage(root, "legs_small.png")
        ImageManager.Images["feet"] = self.loadImage(root, "feet_small.png")
        ImageManager.Images["ring1"] = self.loadImage(root, "ring_small.png")
        ImageManager.Images["ring2"] = self.loadImage(root, "ring_small.png")
        ImageManager.Images["necklace"] = self.loadImage(root, "necklace.png")
        ImageManager.Images["bracelets"] = self.loadImage(root, "bracelet.png")

    def loadImage(self, root, path):
        image = None
        try:
            image = pygame.image.load(os.path.join(root, path)).convert_alpha()
        except:
            image = pygame.image.load(os.path.join(root, "sword_small.png")).convert_alpha()
        return image


import pygame
from LoreleiClient.UIObjects.Input import InputBox, Notification
from LoreleiLib.Packets.Common import PlayerQuitPacket

class UIView(object):

    def __init__(self, makeView, viewType, connection=None, background=(50, 50, 50)):
        self.viewType = viewType
        self.connection = connection
        self.running = True
        self.ui_objects = []
        self.ui_handle_event = []
        self.ui_update = []
        self.background_color = background
        self.notification = None

        self.input_fields = []
        self.input_index = 0

        makeView()

    def add_ui_object(self, ui_obj):
        self.ui_objects.append(ui_obj)
        if hasattr(ui_obj, 'handle_event'):
            self.ui_handle_event.append(ui_obj)
        if hasattr(ui_obj, 'update'):
            self.ui_update.append(ui_obj)
        if isinstance(ui_obj, InputBox):
            self.input_fields.append(ui_obj)

    def remove_ui_object(self, ui_obj):
        if ui_obj in self.ui_objects:
            self.ui_objects.remove(ui_obj)
            if hasattr(ui_obj, 'handle_event'):
                self.ui_handle_event.remove(ui_obj)
            if hasattr(ui_obj, 'update'):
                self.ui_update.remove(ui_obj)
            if isinstance(ui_obj, InputBox):
                self.input_fields.remove(ui_obj)

    def removeAll(self):
        for item in self.ui_objects:
            del item
        for item in self.ui_handle_event:
            del item
        for item in self.ui_update:
            del item
        for item in self.input_fields:
            del item

        self.ui_objects = []
        self.ui_handle_event = []
        self.ui_update = []
        self.input_fields = []

    def tab_event(self):
        if len(self.input_fields) > 0:
            if self.input_index >= 0:
                self.input_fields[self.input_index].set_active(False)

            self.input_index += 1
            if self.input_index >= len(self.input_fields):
                self.input_index = 0

            self.input_fields[self.input_index].set_active(True)

    def handle_events(self, events):
        if not self.running:
            pygame.quit()
            return

        for event in events:
            # Process input events
            if event.type == pygame.QUIT:
                self.connection.sendLine(PlayerQuitPacket())
                self.running = False
            elif event.type == pygame.KEYDOWN:
                if event.key == pygame.K_TAB:
                    self.tab_event()
                    continue

        for event in events:
            for obj in self.ui_handle_event:
                obj.handle_event(event)

        for obj in self.ui_update:
            obj.update()

    def draw(self, screen):
        screen.fill(self.background_color)

        for obj in self.ui_objects:
            obj.draw(screen)

    def handle_data(self, data):
        pass


import pygame
from LoreleiClient.Views.Login import LoginView
from LoreleiClient.Settings.Screen import ScreenSettings
from LoreleiLib.Packets.View import ViewPacket, ViewType
from LoreleiClient.Views.CharacterCreation import CharacterCreationView
from LoreleiClient.Views.Game import GameView


class UI:

    def __init__(self, connection):
        self.running = True
        self.connection = connection
        self.connector = None
        self.DESIRED_FPS = 30.0
        self.SCREEN_WIDTH = ScreenSettings.SCREEN_WIDTH
        self.SCREEN_HEIGHT = ScreenSettings.SCREEN_HEIGHT
        self.screen = pygame.display.set_mode((self.SCREEN_WIDTH, self.SCREEN_HEIGHT))
        self.view = LoginView()
        pygame.display.set_caption("Lorelei")

    def game_tick(self):
        try:
            self.view.draw(self.screen)
            pygame.display.flip()
            self.view.handle_events(pygame.event.get())
            return True
        except Exception as err:
            if err.message.lower() != "display surface quit":
                print err
            return False

    def handle_data(self, data):
        if isinstance(data, ViewPacket):
            if self.view.viewType != data.viewType:
                if data.viewType == ViewType.Login:
                    if type(self.view) != type(LoginView):
                        self.setView(LoginView(self.connection))
                if data.viewType == ViewType.CharacterCreation:
                    if type(self.view) != type(CharacterCreationView):
                        self.setView(CharacterCreationView(self.connection))
                if data.viewType == ViewType.Game:
                    if type(self.view) != type(GameView):
                        self.setView(GameView(self.connection))

        newView = self.view.handle_data(data)
        if newView is not None:
            self.setView(newView)

    def setView(self, view):
        print "Change View"
        self.view = view
        self.view.connection = self.connection
        if hasattr(self.view, 'getData'):
            self.view.getData()

    def set_connection(self, connection):
        self.connection = connection
        self.view.connection = connection

    def close(self):
        pygame.quit()
        self.running = False
        self.connection.loseConnection()


from setuptools import setup, find_packages


def readme():
    with open('README.rst') as f:
        return f.read()


setup(name='LoreleiClient',
    version='0.1.7.0', # Don't forget to make a changelog entry in README.rst
    description='The text based MUD made by Lorinthio',
    author='Lorinthio',
    author_email='benrandallswg@gmail.com',
    long_description=readme(),
    packages=find_packages(),
    install_requires=[
        'twisted',
        'pygame',
        'enum34',
        'LoreleiLib'
    ],
    include_package_data = True,
    package_data = {
    '' : ['*.png'],
    },
    zip_safe=False)


from LoreleiClient.UIObjects.Input import InputBox
from LoreleiLib.Packets.Common import PlayerQuitPacket
import pygame


class Component(object):

    def __init__(self, x, y, makeView):
        self.x = x
        self.y = y
        self.ui_objects = []
        self.ui_handle_event = []
        self.ui_update = []
        self.input_fields = []
        makeView()

    def add_ui_object(self, ui_obj):
        self.ui_objects.append(ui_obj)
        if hasattr(ui_obj, 'handle_event'):
            self.ui_handle_event.append(ui_obj)
        if hasattr(ui_obj, 'update'):
            self.ui_update.append(ui_obj)
        if isinstance(ui_obj, InputBox):
            self.input_fields.append(ui_obj)

    def remove_ui_object(self, ui_obj):
        if ui_obj in self.ui_objects:
            self.ui_objects.remove(ui_obj)
            if hasattr(ui_obj, 'handle_event'):
                self.ui_handle_event.remove(ui_obj)
            if hasattr(ui_obj, 'update'):
                self.ui_update.remove(ui_obj)
            if isinstance(ui_obj, InputBox):
                self.input_fields.remove(ui_obj)

    def removeAll(self):
        for item in self.ui_objects:
            del item
        for item in self.ui_handle_event:
            del item
        for item in self.ui_update:
            del item
        for item in self.input_fields:
            del item

        self.ui_objects = []
        self.ui_handle_event = []
        self.ui_update = []
        self.input_fields = []

    def handle_event(self, event):
        # Process input events
        if event.type == pygame.KEYDOWN:
            if event.key == pygame.K_TAB:
                self.tab_event()

        for obj in self.ui_handle_event:
            obj.handle_event(event)

        for obj in self.ui_update:
            obj.update()

    def tab_event(self):
        if len(self.input_fields) > 0:
            if self.input_index >= 0:
                self.input_fields[self.input_index].set_active(False)

            self.input_index += 1
            if self.input_index >= len(self.input_fields):
                self.input_index = 0

            self.input_fields[self.input_index].set_active(True)

    def draw(self, screen):
        for obj in self.ui_objects:
            obj.draw(screen)


from Common import UIView
from LoreleiClient.UIObjects.Input import InputBox, Button, Notification, LabelRightAligned, PasswordInput, \
    ConditionalUI
from LoreleiLib.Packets.Login import LoginAttemptRequest, LoginAttemptResponse, AccountCreateRequest, \
    AccountCreateResponse
from CharacterCreation import CharacterCreationView
from LoreleiClient.Settings.Screen import ScreenSettings
from LoreleiLib.Packets.View import ViewType


class LoginView(UIView):

    def __init__(self, connection=None):
        # Type: () -> None
        width = ScreenSettings.SCREEN_WIDTH
        height = ScreenSettings.SCREEN_HEIGHT
        self.username_label = LabelRightAligned(width/2 - 140, height/2-70, 80, 20, "Username : ", fontSize=24)
        self.username_box = InputBox(width/2-60, height/2-70, 200, 26, fontSize=24)
        self.password_label = LabelRightAligned(width/2 - 140, height/2-35, 80, 20, "Password : ", fontSize=24)
        self.password_box = PasswordInput(width/2-60, height/2-35, 200, 26, fontSize=24)
        self.create_button = Button(width/2-60, height/2, 60, 30, "Create", self.create, fontSize=24)
        self.login_button = Button(width/2+20, height/2, 60, 30, "Login", self.login, fontSize=24)
        self.connecting = ConditionalUI(Notification("Connecting...", hideButton=True), self.checkConnection)
        super(LoginView, self).__init__(self.make_view, ViewType.Login, connection)

    def make_view(self):
        self.add_ui_object(self.username_label)
        self.add_ui_object(self.username_box)
        self.add_ui_object(self.password_label)
        self.add_ui_object(self.password_box)
        self.add_ui_object(self.login_button)
        self.add_ui_object(self.create_button)
        self.add_ui_object(self.create_button)
        self.add_ui_object(self.connecting)

    def checkConnection(self):
        #print "Check Connection"
        if self.connection is None:
            return True
        return False

    def notificationCallback(self):
        self.remove_ui_object(self.notification)

    def login(self):
        self.connection.sendLine(LoginAttemptRequest(self.username_box.text, self.password_box.text))

    def create(self):
        self.connection.sendLine(AccountCreateRequest(self.username_box.text, self.password_box.text))

    def handle_data(self, data):
        print "Got,", data
        if isinstance(data, LoginAttemptResponse):
            if not data.success:
                self.add_ui_object(Notification(data.message, self.notificationCallback))
        elif isinstance(data, AccountCreateResponse):
            self.add_ui_object(Notification(data.message, self.notificationCallback))
        return None


from LoreleiClient.Views.Modules.Common import Component
from LoreleiClient.UIObjects.Input import Box, Label, LabelRightAligned, ObservableValue, IntValueBox


class AttributeComponent(Component):

    def __init__(self, x, y, character):
        self.character = character
        self.strength = ObservableValue(self.character.attributes.strength)
        self.constitution = ObservableValue(self.character.attributes.constitution)
        self.dexterity = ObservableValue(self.character.attributes.dexterity)
        self.agility = ObservableValue(self.character.attributes.agility)
        self.wisdom = ObservableValue(self.character.attributes.wisdom)
        self.intelligence = ObservableValue(self.character.attributes.intelligence)
        self.charisma = ObservableValue(self.character.attributes.charisma)
        super(AttributeComponent, self).__init__(x, y, self.setup)

    def setup(self):
        labelX = self.x + 40
        intValueX = labelX + 88
        spacing = 32
        y = self.y
        self.add_ui_object(Box(self.x, y, 180, 240, 3, "Attributes", fontSize=24))
        y += 10
        self.add_ui_object(LabelRightAligned(labelX, y, 80, 26, "Strength :", fontSize=24))
        self.add_ui_object(IntValueBox(intValueX, y, 30, 26, self.strength, fontSize=24))
        y += spacing
        self.add_ui_object(LabelRightAligned(labelX, y, 80, 26, "Constitution :", fontSize=24))
        self.add_ui_object(IntValueBox(intValueX, y, 30, 26,  self.constitution, fontSize=24))
        y += spacing
        self.add_ui_object(LabelRightAligned(labelX, y, 80, 26, "Dexterity :", fontSize=24))
        self.add_ui_object(IntValueBox(intValueX, y, 30, 26,  self.dexterity, fontSize=24))
        y += spacing
        self.add_ui_object(LabelRightAligned(labelX, y, 80, 26, "Agility :", fontSize=24))
        self.add_ui_object(IntValueBox(intValueX, y, 30, 26,  self.agility, fontSize=24))
        y += spacing
        self.add_ui_object(LabelRightAligned(labelX, y, 80, 26, "Wisdom :", fontSize=24))
        self.add_ui_object(IntValueBox(intValueX, y, 30, 26,  self.wisdom, fontSize=24))
        y += spacing
        self.add_ui_object(LabelRightAligned(labelX, y, 80, 26, "Intelligence :", fontSize=24))
        self.add_ui_object(IntValueBox(intValueX, y, 30, 26,  self.intelligence, fontSize=24))
        y += spacing
        self.add_ui_object(LabelRightAligned(labelX, y, 80, 26, "Charisma :", fontSize=24))
        self.add_ui_object(IntValueBox(intValueX, y, 30, 26,  self.charisma, fontSize=24))

    def update(self):
        self.strength.setValue(self.character.attributes.strength)
        self.constitution.setValue(self.character.attributes.constitution)
        self.dexterity.setValue(self.character.attributes.dexterity)
        self.agility.setValue(self.character.attributes.agility)
        self.wisdom.setValue(self.character.attributes.wisdom)
        self.intelligence.setValue(self.character.attributes.intelligence)
        self.charisma.setValue(self.character.attributes.charisma)